#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import scipy.stats

#======================================
#========== set param
figSave = 'off'

# you can change the following biological parameters for a sensitivity test
#= initial cell concentration
aexp_X0 = 6.33 # Aexp X0 1~1000

#= algal growth rate
aexp_Gr = 0.016 # Aexp Gr 0.010~0.025

#= Onuma et al., 2018 reported aexp_X0 = 6.33 or 694
#= Onuma et al., 2018 reported aexp_Gr = 0.016 or 0.018

doy_min, doy_max = 0,365	# north hemisphere

#======================================
#========== data list

FD_List = ['gswp3','wfdei','crujra']	# reanalysis dataset

site_list = ['Feiringbreen','QA_SB','Werenskioldbreen','Mittivakkat','Gulkana','No.33','Harding',
		 'Akkem','Tiefenbach','UM_No1','TigoaPass','TA_Rai','Philistine','Tyndall','Livingston']
#= carrying capacity at different locations
K_list =    [2.0e8         ,3.5e7  ,5.9e9             ,1.8e8        ,5.1e7    ,6.9e6  ,6.7e9  ,
		 2.2e6  ,5.0e8       ,1.3e6   ,1.3e7      ,1.1e8   ,1.2e7       ,6.0e5    ,4.9e8       ]
year_list = [2013          ,2014   ,2004              ,2012         ,2001     ,2012   ,2000   ,
		 2002   ,2008        ,2013    ,2000       ,2012    ,1998        ,1999     ,2015        ]
sName_list =['SV-S'        ,'GL-Q','SV-W'             ,'GL-A'       ,'AK-E'   ,'SB-S' ,'AK-K' ,
		 'AT-A' ,'EU-T'      ,'TN-U'  ,'NA-S'     ,'JP-T'  ,'NZ-A'      ,'PG-T'   ,'AP-L']
doy_list =  [220           ,214   ,202                ,192          ,218      ,191      ,213   ,
		 199    ,255         ,238     ,200        ,167     ,358         ,319      ,26        ]
sColor_list = ['red'         ,'sienna','orange'         ,'gold'   ,'yellowgreen','lime'   ,'mediumturquoise' ,
		 'cyan' ,'dodgerblue','lightsteelblue','navy' ,'darkviolet','magenta'   ,'pink'   ,'dimgray' ]

#======================================
#========== main
def main():
	#===== set path
	indir = '../data'
	outdir = '../figure'

	#===== read csv data
	print'### read obs data (csv) ###'
	doy,cells,cells_sd = make_list(len(site_list))
	lon,lat,elv = make_list(len(site_list))
	doy_l,cells_l,cells_sd_l = make_list(len(site_list))
	for n in range(len(site_list)):
		doy[n],cells[n],cells_sd[n],lon[n],lat[n],elv[n], \
		doy_l[n],cells_l[n],cells_sd_l[n] \
		= rd_obs(site_list[n],year_list[n])

	#===== make list for sites loop
	doy_m1,doy_m2,doy_m3 = make_list(len(site_list))	# FD1-3
	mlt2_FD1,mlt3_FD1,mlt4_FD1 = make_list(len(site_list))	# FD1
	mlt2_FD2,mlt3_FD2,mlt4_FD2 = make_list(len(site_list))	# FD2
	mlt2_FD3,mlt3_FD3,mlt4_FD3 = make_list(len(site_list))	# FD3
	acc_mlt2_FD1,acc_mlt3_FD1,acc_mlt4_FD1 = make_list(len(site_list))	# FD1
	acc_mlt2_FD2,acc_mlt3_FD2,acc_mlt4_FD2 = make_list(len(site_list))	# FD2
	acc_mlt2_FD3,acc_mlt3_FD3,acc_mlt4_FD3 = make_list(len(site_list))	# FD3
	rab_mlt2_FD1,rab_mlt3_FD1,rab_mlt4_FD1 = make_list(len(site_list))	# FD1
	rab_mlt2_FD2,rab_mlt3_FD2,rab_mlt4_FD2 = make_list(len(site_list))	# FD2
	rab_mlt2_FD3,rab_mlt3_FD3,rab_mlt4_FD3 = make_list(len(site_list))	# FD3

	#=== read melt hour2,3,4 (csv)
	for n in range(len(site_list)):
		doy_m1[n],mlt2_FD1[n],mlt3_FD1[n],mlt4_FD1[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[0])
		doy_m2[n],mlt2_FD2[n],mlt3_FD2[n],mlt4_FD2[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[1])
		doy_m3[n],mlt2_FD3[n],mlt3_FD3[n],mlt4_FD3[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[2])


	#=== cal algal cell conc.
	# acc_I1[Site][X0], acc_G1[Site][Gr], acc_A1[Site][2]	# FD1
	for n in range(len(site_list)):
		print 'cal %s (%d)'%(site_list[n],year_list[n])
		acc_mlt2_FD1[n],rab_mlt2_FD1[n] = cal_acc(mlt2_FD1[n],FD_List[0],K_list[n],site_list[n],'Aexp')
		acc_mlt3_FD1[n],rab_mlt3_FD1[n] = cal_acc(mlt3_FD1[n],FD_List[0],K_list[n],site_list[n],'Aexp')
		acc_mlt4_FD1[n],rab_mlt4_FD1[n] = cal_acc(mlt4_FD1[n],FD_List[0],K_list[n],site_list[n],'Aexp')
		acc_mlt2_FD2[n],rab_mlt2_FD2[n] = cal_acc(mlt2_FD2[n],FD_List[1],K_list[n],site_list[n],'Aexp')
		acc_mlt3_FD2[n],rab_mlt3_FD2[n] = cal_acc(mlt3_FD2[n],FD_List[1],K_list[n],site_list[n],'Aexp')
		acc_mlt4_FD2[n],rab_mlt4_FD2[n] = cal_acc(mlt4_FD2[n],FD_List[1],K_list[n],site_list[n],'Aexp')
		acc_mlt2_FD3[n],rab_mlt2_FD3[n] = cal_acc(mlt2_FD3[n],FD_List[2],K_list[n],site_list[n],'Aexp')
		acc_mlt3_FD3[n],rab_mlt3_FD3[n] = cal_acc(mlt3_FD3[n],FD_List[2],K_list[n],site_list[n],'Aexp')
		acc_mlt4_FD3[n],rab_mlt4_FD3[n] = cal_acc(mlt4_FD3[n],FD_List[2],K_list[n],site_list[n],'Aexp')

	#=== plot figure
	#= FDs acc_mlt_FD1[mlt][site][X]
	acc_mlt_FD1 = [acc_mlt2_FD1,acc_mlt3_FD1,acc_mlt4_FD1]
	acc_mlt_FD2 = [acc_mlt2_FD2,acc_mlt3_FD2,acc_mlt4_FD2]
	acc_mlt_FD3 = [acc_mlt2_FD3,acc_mlt3_FD3,acc_mlt4_FD3]
	print [len(v) for v in acc_mlt_FD1]

	#= FDs mlt_FD1[mlt][site]
	mlt_FD1 = [mlt2_FD1,mlt3_FD1,mlt4_FD1]
	mlt_FD2 = [mlt2_FD2,mlt3_FD2,mlt4_FD2]
	mlt_FD3 = [mlt2_FD3,mlt3_FD3,mlt4_FD3]
	print [len(v) for v in mlt_FD1]
	
	#=== figure (mlthr)
	#plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,mlt_FD1,mlt_FD2,mlt_FD3,FD_List[0],"mlt")
	plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,mlt_FD1,mlt_FD2,mlt_FD3,FD_List[1],"mlt")
	#plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,mlt_FD1,mlt_FD2,mlt_FD3,FD_List[2],"mlt")

	#=== figure (acc)
	#plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,acc_mlt_FD1,acc_mlt_FD2,acc_mlt_FD3,FD_List[0],"acc")
	plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,acc_mlt_FD1,acc_mlt_FD2,acc_mlt_FD3,FD_List[1],"acc")
	#plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m2,acc_mlt_FD1,acc_mlt_FD2,acc_mlt_FD3,FD_List[2],"acc")

	#=== figure (rab)
	#plot_rtab(outdir,site_list,rab_mlt2_FD1,rab_mlt3_FD1,rab_mlt4_FD1,FD_List[0])
	plot_rtab(outdir,site_list,rab_mlt2_FD2,rab_mlt3_FD2,rab_mlt4_FD2,FD_List[1])
	#plot_rtab(outdir,site_list,rab_mlt2_FD3,rab_mlt3_FD3,rab_mlt4_FD3,FD_List[2])

#======================================
#===== read observational data (CSV)
def rd_obs(site_obs,year_obs):
	Tab_list = ['Ref','Glacier','Species','Year_obs','DOY_obs','lon_obs','lat_obs','elv_obs','cells_ave_obs','cells_sd_obs']

	obs_file = '../data/S_nivaloides_obs.csv'
	df = pd.read_csv(obs_file,encoding='UTF-8')

	#=== read csv data
	glacier = df[Tab_list[1]].values.tolist()
	lon_obs = df[Tab_list[5]].values.tolist()
	lat_obs = df[Tab_list[6]].values.tolist()
	elv_obs = df[Tab_list[7]].values.tolist()
	
	site_idx = glacier.index(site_obs)
	lon_obs = lon_obs[site_idx]
	lat_obs = lat_obs[site_idx]
	elv_obs = float(elv_obs[site_idx])

	#=== read algal cell abundance
	print site_obs
	if site_obs in ['QA_S2.5','QA_SB','TA_Muro','TA_Rai','UM_No1','Gulkana','Livingston']:
		obs_file_season = '../data/S.nivaloides_%s_%d.csv'%(site_obs,year_obs)
		df_season = pd.read_csv(obs_file_season,encoding='UTF-8')
		doy_obs = df_season["doy"]
		cells_obs = df_season["typeA_ave"]
		print np.max(cells_obs)
		cells_obs_sd = df_season["typeA_sd"]

		doy_obs_log = df_season["doy_log"]
		cells_obs_log = df_season["typeA_ave_log"]
		cells_obs_sd_log = df_season["typeA_sd_log"]
	else:
		doy_obs = df[Tab_list[4]].values.tolist()
		cells_obs = df[Tab_list[8]].values.tolist()
		cells_obs_sd = df[Tab_list[9]].values.tolist()
		doy_obs = doy_obs[site_idx]
		cells_obs = cells_obs[site_idx]
		cells_obs_sd = cells_obs_sd[site_idx]
		doy_obs_log = doy_obs
		cells_obs_log = cells_obs
		cells_obs_sd_log = cells_obs_sd

	print "### %s ###"%site_obs
	print "lat %s, lon %s, elv %.2f m"%(lat_obs,lon_obs,elv_obs)
	print doy_obs

	return doy_obs,cells_obs,cells_obs_sd,lon_obs,lat_obs,elv_obs, \
		doy_obs_log,cells_obs_log,cells_obs_sd_log

#======================================
#===== make list for variables roop
def make_list(N):

	acc1,acc2,acc3 = [],[],[]

	for i in range(N):
		acc1.append(i)
		acc2.append(i)
		acc3.append(i)

	return acc1,acc2,acc3

#======================================
#===== set path & read model data
def rd_csv(indir,site,year,FD):
	import datetime

	Tab_list = ['Date','mlthr2','mlthr3','mlthr4']

	infile = '%s/mlthr_%d_%s_%s.csv'%(indir,year,site,FD)

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		doy,mlt2,mlt3,mlt4 = -999,-999,-999,-999
		return doy,mlt2,mlt3,mlt4

	#=== read csv data
	N1 = Tab_list.index('Date')
	Date = df[Tab_list[N1]].values.tolist()
	N2 = Tab_list.index('mlthr2')
	mlt2 = df[Tab_list[N2]].values.tolist()
	N3 = Tab_list.index('mlthr3')
	mlt3 = df[Tab_list[N3]].values.tolist()
	N4 = Tab_list.index('mlthr4')
	mlt4 = df[Tab_list[N4]].values.tolist()

	Date = Date[1::]
	mlt2 = mlt2[1::]
	mlt3 = mlt3[1::]
	mlt4 = mlt4[1::]

	#=== unicode -> float
	mlt2_f = np.zeros(len(mlt2))
	mlt3_f = np.zeros(len(mlt3))
	mlt4_f = np.zeros(len(mlt4))

	for t in range(len(mlt2)):
		mlt2_f[t] = float(mlt2[t])
		mlt3_f[t] = float(mlt3[t])
		mlt4_f[t] = float(mlt4[t])

	#=== date -> doy
	doy = []
	for i in range(len(Date)):
		tmp = str(Date[i]).replace(' ','')
		tmp = datetime.datetime.strptime(tmp,'%Y%m%d%H%M%S')
		doy.append((tmp - datetime.datetime(year,1,1,12)).days + 1)  # cal doy

	return doy,mlt2_f,mlt3_f,mlt4_f

#======================================
#===== cal algal cell concentration
def cal_acc(mlthr,FD,K,site,exp):

	#if mlthr == -999:
	if type(mlthr) == int:
		print "%s (%s) no data"%(exp,FD)
		acc_all,rab_all = -999,-999
		return acc_all,rab_all

	print "%s start (%s)"%(exp,FD)

	#=== sensitivity tests
	if exp == 'Iexp':
		acc_all = copy.deepcopy(X0_list)
		rab_all = copy.deepcopy(acc_all)
		for x in range(len(X0_list)):
			acc_all[x],rab_all[x] = logi(mlthr,X0_list[x],0.0180,site,K)

	elif exp == 'Gexp':
		acc_all = copy.deepcopy(Gr_list)
		rab_all = copy.deepcopy(acc_all)
		for g in range(len(Gr_list)):
			acc_all[g],rab_all[g] = logi(mlthr,6.33,Gr_list[g],site,K)
	elif exp == 'Aexp':
		acc_all,rab_all = logi(mlthr,aexp_X0,aexp_Gr,site,K)

	return acc_all,rab_all

#======================================
#===== logistic model
def logi(mlthr,X0,Gr,site,K):

	acc = np.zeros(len(mlthr))

	for t in range(len(mlthr)):
		acc[t] = K / ( 1 + (( K - X0 ) / X0 ) * ( np.exp(1) ** ( Gr * ( float(mlthr[t]) * -1 ))))

	#=== cal reaching time to the algal blooming
	rab = [i for i, x in enumerate(acc) if x > 5.e5]

	if not rab:
		# no appearance of the algal blooming (<5.e5) 
		rab = acc.tolist().index(np.max(acc))
	else:
		# algal blooming at first doy
		rab = np.min(rab)

	if site in ['Philistine','Tyndall']:
		rab = rab + 179

	return acc, rab

#======================================
#===== cal correlation
def cal_corr(doy_obs,cells_obs,data):
	data_corr = []

	for n in range(len(doy_obs)):	
		data_corr = np.append(data_corr,data[int(doy_obs[n]+1)])

	R,P = scipy.stats.pearsonr(cells_obs,data_corr)
	R2 = R**2
	print 'R, P, R^2 (vs. obs)= ',R,P,R2

	return R,P,R2

#======================================
#===== plot figure
def plot(outdir,site_list,lon,lat,elv,doy,cells,cells_sd,doy_l,cells_l,cells_sd_l,year_list,doy_m,d_FD1,d_FD2,d_FD3,FD,Var):


	#=== set plot param
	fig, axes = plt.subplots(5, 3, figsize=(16,14))
	plt.subplots_adjust(wspace=0.15,hspace=0.60) # adjust plot space

	C1 = "orangered"
	C2 = "lime"
	C3 = "dodgerblue"

	#=== plot model result
	if Var == 'mlt':
		label1 = '$GP^{M}$ (Snow melt)'
		label2 = '$GP^{MD}$ (Snow melt + Daylight length)'
		label3 = '$GP^{MDF}$ (Snow melt + Daylight length + Snowfall)'
		y_min, y_max = 0, 3000
		txt_y = 1350
	elif Var == 'acc':
		label1 = '$X^{M}$ (Snow melt)'
		label2 = '$X^{MD}$ (Snow melt + Daylight length)'
		label3 = '$X^{MDF}$ (Snow melt + Daylight length + Snowfall)'
		y_min, y_max = 1e0, 1e11 	# log scale (X=694)
		txt_y = 1e5

	i,j = 0,0
	for n in range(len(site_list)):

		#== set axis
		axes[i][j].set_xlim(xmin=doy_min)
		axes[i][j].set_xlim(xmax=doy_max)
		xlabels = np.arange(doy_min+10,doy_max+1,30)
		axes[i][j].set_xticks(xlabels)
		axes[i][j].set_xlabel('Day of the Year (%d)'%year_list[n],fontsize=10)
		if j == 0 and Var == 'mlt':
			axes[i][j].set_ylabel('Growth period\n(hours)',fontsize=11)
		elif j == 0 and Var == 'acc':
			axes[i][j].set_ylabel('Algal cell concentration\n(cells $\mathregular{m^{-2}}$)',fontsize=11)
		if Var == 'acc':
			axes[i][j].set_yscale("log")
		axes[i][j].set_ylim(ymin=y_min)
		axes[i][j].set_ylim(ymax=y_max)
		axes[i][j].set_title(sName_list[n],fontsize=15)

		#== plot results (no data skip)
		print site_list[n]
		if site_list[n] == 'Livingston':
			doy_1 = 0
			mlt2_1 = 0 #d_FD1[0][n]
			mlt3_1 = 0 #d_FD1[1][n]
			mlt4_1 = 0 #d_FD1[2][n]
			doy_3 = 0
			mlt2_3 = 0 #d_FD3[0][n]
			mlt3_3 = 0 #d_FD3[1][n]
			mlt4_3 = 0 #d_FD3[2][n]
			if Var == 'mlt':
				doy_2 = doy_m[n]
				mlt2_2 = d_FD2[0][n]
				mlt3_2 = d_FD2[1][n]
				mlt4_2 = d_FD2[2][n]
			elif Var == 'acc':
				doy_2 = doy_m[n]
				mlt2_2 = d_FD2[0][n]
				mlt3_2 = d_FD2[1][n]
				mlt4_2 = d_FD2[2][n]
		elif Var in ['mlt','acc']:
			doy_1 = doy_m[n]
			mlt2_1 = d_FD1[0][n]
			mlt3_1 = d_FD1[1][n]
			mlt4_1 = d_FD1[2][n]
			doy_2 = doy_m[n]
			mlt2_2 = d_FD2[0][n]
			mlt3_2 = d_FD2[1][n]
			mlt4_2 = d_FD2[2][n]
			doy_3 = doy_m[n]
			mlt2_3 = d_FD3[0][n]
			mlt3_3 = d_FD3[1][n]
			mlt4_3 = d_FD3[2][n]

		if FD == "gswp3":
			axes[i][j].plot(doy_1,mlt2_1,color=C1,linestyle='-',lw=2,label=label1)
			axes[i][j].plot(doy_1,mlt3_1,color=C2,linestyle='-',lw=2,label=label2)
			axes[i][j].plot(doy_1,mlt4_1,color=C3,linestyle='-',lw=2,label=label3)
			if site_list[n] == 'Livingston':
				axes[-1][-1].text(doy_min+130,txt_y,'no data',color='black',size=20)
		elif FD == "wfdei":
			axes[i][j].plot(doy_2,mlt2_2,color=C1,linestyle='-',lw=2,label=label1)
			axes[i][j].plot(doy_2,mlt3_2,color=C2,linestyle='-',lw=2,label=label2)
			axes[i][j].plot(doy_2,mlt4_2,color=C3,linestyle='-',lw=2,label=label3)
		elif FD == "crujra":
			axes[i][j].plot(doy_3,mlt2_3,color=C1,linestyle='-',lw=2,label=label1)
			axes[i][j].plot(doy_3,mlt3_3,color=C2,linestyle='-',lw=2,label=label2)
			axes[i][j].plot(doy_3,mlt4_3,color=C3,linestyle='-',lw=2,label=label3)
			if site_list[n] == 'Livingston':
				axes[-1][-1].text(doy_min+130,txt_y,'no data',color='black',size=20)


		j = j + 1
		if j == 3:
			i = i + 1
			j = 0

	
	if Var == "acc":
	#=== plot obs result
		i,j = 0,0
		for n in range(len(site_list)):
			print n

		#== no model data skip
			if site_list[n] == 'Livingston' and FD in ['gswp3','crujra']:
				axes[i][j].plot(0,color="black",lw=1.5,label='Observation')
				continue

		#=== set error bar
			if type(cells[n]) is int or float :
#
#				#= 1 point obs data
				lowerER = [0,0]
				upperER = [0,cells_sd_l[n]]
				doy_obs = [doy_l[n],doy_l[n]]
				cells_obs = [cells_l[n],cells_l[n]]
				#=
				axes[i][j].set_ylim(ymin=1e0)
				axes[i][j].set_ylim(ymax=1e11)
				axes[i][j].set_yscale("log")

				axes[i][j].errorbar(doy_obs,cells_obs,yerr=[lowerER,upperER],
					color="black",ls='',lw=1.5,markersize=3,marker='o',ecolor='black',capsize=2,label='Observation')
				axes[i][j].axhline(y=5.0e5,color='grey',ls='--')

				#=== corr
				if site_list[n] == 'UM_No1':
					doy[n] = doy[n][0:5]
					cells[n] = cells[n][0:5]
				if FD == 'gswp3' and site_list[n] in ['QA_SB','Gulkana','UM_No1','TA_Rai','Livingston']:
					R_2,P_2,R2_2 = cal_corr(doy[n],cells[n],d_FD1[0][n])	# mlt2
					R_3,P_3,R2_3 = cal_corr(doy[n],cells[n],d_FD1[1][n])
					R_4,P_4,R2_4 = cal_corr(doy[n],cells[n],d_FD1[2][n])
				elif FD == 'wfdei' and site_list[n] in ['QA_SB','Gulkana','UM_No1','TA_Rai','Livingston']:
					R_2,P_2,R2_2 = cal_corr(doy[n],cells[n],d_FD2[0][n])	# mlt2
					R_3,P_3,R2_3 = cal_corr(doy[n],cells[n],d_FD2[1][n])
					R_4,P_4,R2_4 = cal_corr(doy[n],cells[n],d_FD2[2][n])
				elif FD == 'crujra' and site_list[n] in ['QA_SB','Gulkana','UM_No1','TA_Rai','Livingston']:
					R_2,P_2,R2_2 = cal_corr(doy[n],cells[n],d_FD3[0][n])	# mlt2
					R_3,P_3,R2_3 = cal_corr(doy[n],cells[n],d_FD3[1][n])
					R_4,P_4,R2_4 = cal_corr(doy[n],cells[n],d_FD3[2][n])

				if site_list[n] in ['QA_SB','Gulkana','UM_No1']:
					#= mlthr2
					#axes[i][j].text(30,2e2,'$R^2$=%01.2f\n(cal vs. obs)'%(R2_2),size=10.5)
					#= mlthr3
					axes[i][j].text(30,3e6,'$R^2$=%01.2f\n($X^{MD}$ vs. Obs)'%(R2_3),size=10.5)
					#= mlthr4
					axes[i][j].text(30,1e2,'$R^2$=%01.2f\n($X^{MDF}$ vs. Obs)'%(R2_4),size=10.5)
				elif site_list[n] in ['TA_Rai']:
					#axes[i][j].text(15,2e2,'$R^2$=%01.2f\n(cal vs. obs)'%(R2_2),size=10.5)
					axes[i][j].text(15,3e6,'$R^2$=%01.2f\n($X^{MD}$ vs. Obs)'%(R2_3),size=10.5)
					axes[i][j].text(15,1e2,'$R^2$=%01.2f\n($X^{MDF}$ vs. Obs)'%(R2_4),size=10.5)
				elif site_list[n] in ['Livingston']:
					#axes[i][j].text(240,2e2,'$R^2$=%01.2f\n(cal vs. obs)'%(R2_2),size=10.5)
					axes[i][j].text(120,5e6,'$R^2$=%01.2f\n($X^{MD}$ vs. Obs)'%(R2_3),size=10.5)
					axes[i][j].text(240,5e6,'$R^2$=%01.2f\n($X^{MDF}$ vs. Obs)'%(R2_4),size=10.5)


			j = j + 1
			if j == 3:
				i = i + 1
				j = 0

#===

	plt.legend(loc='lower center', bbox_to_anchor=(-0.65,-0.85),ncol=4,fontsize=14)

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s234_15sites_%s.png'%(outdir,Var,FD),bbox_inches="tight",dpi=300)
		print 'save %s/%s234_15sites_%s.png'%(outdir,Var,FD)
	else:
		print '%s/%s234_15sites_%s.png'%(outdir,Var,FD)
		plt.show()
	plt.close()


#======================================
#===== plot figure (rtab obs vs. cal)
def plot_rtab(outdir,site_list,d_m2,d_m3,d_m4,FD):

	#=== set plot param
	fig, axes = plt.subplots(1, 1, figsize=(11,10))
	plt.tick_params(labelsize=16)

	for n in range(len(site_list)):
		#print doy_list[n],d_m2[n],d_m3[n],d_m4[n]
		axes.plot(d_m2[n],doy_list[n],color=sColor_list[n],marker='x',markersize=10)
		axes.plot(d_m4[n],doy_list[n],color=sColor_list[n],marker='o',markersize=10,label=sName_list[n])


	#=== set label
	axes.set_ylim(ymin=10, ymax=366)
	axes.set_xlim(xmin=10, xmax=366)
	axes.set_ylabel('Date of the maximum algal cell concentration (Obs)',fontsize=18)
	axes.set_xlabel('Reaching time to the algal bloom (RTAB, Sim)',fontsize=18)

	axes.text(25,315,"$\it{X_{0}}$ = %s cells $\mathregular{m^{-2}}$"%aexp_X0,fontsize=20)
	axes.text(25,295,"$\it{\mu}$ = %s $\mathregular{hour^{-1}}$"%aexp_Gr,fontsize=20)

	if aexp_X0 == 6.33 and aexp_Gr == 0.016:
		axes.text(20,345,"(a)", fontsize=24)
		figName = 'rtab_mlt2vs4_15sites_%s_a'%(FD)
	elif aexp_X0 == 694 and aexp_Gr == 0.018:
		axes.text(20,345,"(b)", fontsize=24)
		figName = 'rtab_mlt2vs4_15sites_%s_b'%(FD)
	else:
		axes.text(20,345,"(x)", fontsize=24)
		figName = 'rtab_mlt2vs4_15sites_%s_x'%(FD)

	plt.legend(loc='center right', fontsize=14)

	#=== set y=x line
	y1 = np.arange(1,367,1) 
	x1 = np.arange(1,367,1) 
	axes.plot(x1,y1,color="black",linestyle="--",lw=2)

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s.png'%(outdir,figName),bbox_inches="tight",dpi=300)
		print 'save %s/%s.png'%(outdir,figName)
	else:
		print '%s/%s.png'%(outdir,figName)
		plt.show()

	plt.close()

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <yonuma613@gmail.com>'
__version__ = '1.2.0'
__date__ = 'Dec. 2021'
