#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import copy

#======================================
#========== set param
figSave = 'off'

#=== X0exp:fig4a, Uexp:fig4b, Fdexp:fig5a
Exp = 'Uexp'	# X0exp, Uexp, Fdexp

Mlt = 'mlthr4'	# growth period (melt+daylight+snowfall)
FD_List = ['gswp3','wfdei','crujra']	# forcing

#=== biological parameters
X0_list = np.arange(1,1002,50).tolist()	# for X0exp
Gr_list = np.arange(0.010,0.0251,0.0005).tolist() # for Uexp
aexp_X0 = 6.33 # for Fdexp

#======================================
#========== main
def main():
	#===== set path
	indir = '../data'
	outdir = '../figure'

	#===== set name list
	set_list(Exp)

	#===== make list for sites loop
	data1,data2,data3 = make_list(len(site_list))	# FD1-3
	doy_m1,doy_m2,doy_m3 = make_list(len(site_list))
	acc_FD1,acc_FD2,acc_FD3 = make_list(len(site_list))	# algal cell concentration
	rab_FD1,rab_FD2,rab_FD3 = make_list(len(site_list))	# rab (reaching time to the algal blooming)
	dab_min_FD1,dab_min_FD2,dab_min_FD3 = make_list(len(site_list))	# dab (duration of the algal blooming)
	dab_max_FD1,dab_max_FD2,dab_max_FD3 = make_list(len(site_list))	# dab (duration of the algal blooming)

	#=== read melt hour (csv)
	for n in range(len(site_list)):
		data1[n],doy_m1[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[0])
		data2[n],doy_m2[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[1])
		data3[n],doy_m3[n] = rd_csv(indir,site_list[n],year_list[n],FD_List[2])

	#=== cal algal cell conc.
	for n in range(len(site_list)):
		print 'cal %s (%d)'%(site_list[n],year_list[n])
		acc_FD1[n],rab_FD1[n] = cal_acc(data1[n],FD_List[0],K_list[n],site_list[n],Exp)
		acc_FD2[n],rab_FD2[n] = cal_acc(data2[n],FD_List[1],K_list[n],site_list[n],Exp)
		acc_FD3[n],rab_FD3[n] = cal_acc(data3[n],FD_List[2],K_list[n],site_list[n],Exp)
		dab_min_FD1[n],dab_max_FD1[n] = cal_dab(acc_FD1[n],rab_FD1[n],site_list[n],Exp)
		dab_min_FD2[n],dab_max_FD2[n] = cal_dab(acc_FD2[n],rab_FD2[n],site_list[n],Exp)
		dab_min_FD3[n],dab_max_FD3[n] = cal_dab(acc_FD3[n],rab_FD3[n],site_list[n],Exp)

	#=== plot figure
	if Exp in ['X0exp','Uexp']:
	#= FD 1
		#plot(outdir,site_list,year_list,acc_FD2,rab_FD2,dab_min_FD2,dab_max_FD2,FD_List[0],Exp)
	#= FD 2
		plot(outdir,site_list,year_list,acc_FD2,rab_FD2,dab_min_FD2,dab_max_FD2,FD_List[1],Exp)
	#= FD 3
		#plot(outdir,site_list,year_list,acc_FD2,rab_FD2,dab_min_FD2,dab_max_FD2,FD_List[2],Exp)
	#= FDs
	elif Exp == 'Fdexp':
		acc_A = [acc_FD1,acc_FD2,acc_FD3]
		rab_A = np.zeros((14)).tolist()
		for site in range(len(site_list)):
			rab_A[site] = [rab_FD1[site],rab_FD2[site],rab_FD3[site]]
		plot(outdir,site_list,year_list,acc_A,rab_A,dab_min_FD2,dab_max_FD2,6.33,Exp)

#======================================
#===== set list for variables roop
def set_list(exp):
	global slist, klist, ylist, nlist, clist, dlist
	global site_list, K_list, year_list, sName_list, sColor_list, doy_list

	slist = ['Feiringbreen','QA_SB','Werenskioldbreen','Mittivakkat','Gulkana','No.33','Harding',
		 'Akkem','Tiefenbach','UM_No1','TigoaPass','TA_Rai','Philistine','Tyndall','Livingston']
	klist = [2.0e8         ,3.5e7  ,5.9e9             ,1.8e8        ,5.1e7    ,6.9e6    ,6.7e9  ,
		 2.2e6  ,5.0e8       ,1.3e6   ,1.3e7      ,1.1e8   ,1.2e7       ,6.0e5    ,4.9e8       ]
	ylist = [2013          ,2014   ,2004              ,2012         ,2001     ,2012     ,2000   ,
		 2002   ,2008        ,2013    ,2000       ,2012    ,1998        ,1999     ,2015        ]
	nlist = ['SV-S'        ,'GL-Q','SV-W'             ,'GL-A'       ,'AK-E'    ,'SB-S'  ,'AK-K' ,
		 'AT-A' ,'EU-T'      ,'TN-U'  ,'NA-S'     ,'JP-T'  ,'NZ-A'      ,'PG-T'   ,'AP-L']
	clist = ['red'         ,'sienna','orange'         ,'gold'   ,'yellowgreen','lime'   ,'mediumturquoise' ,
		 'cyan' ,'dodgerblue','lightsteelblue','navy' ,'darkviolet','magenta'   ,'pink'   ,'dimgray' ]
	dlist = [220           ,214   ,202                ,192          ,218      ,191      ,213   ,
		 199    ,255         ,238     ,200        ,167     ,358         ,319      ,26        ]

	s_tmp = copy.deepcopy(slist)
	k_tmp = copy.deepcopy(klist)
	y_tmp = copy.deepcopy(ylist)
	n_tmp = copy.deepcopy(nlist)
	c_tmp = copy.deepcopy(clist)
	d_tmp = copy.deepcopy(dlist)

	if exp == 'X0exp':
		rmlist = [0,0,0,1,0,0,0,0,0,0,0,0,0,0,1]
	elif exp == 'Uexp':
		rmlist = [0,0,0,1,0,1,0,0,1,0,0,0,1,0,1]
	elif exp == 'Fdexp':
		rmlist = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,1]

	for n in range(len(rmlist)-1,-1,-1):
		if rmlist[n] == 1:
			s_tmp.pop(n)
			k_tmp.pop(n)
			y_tmp.pop(n)
			n_tmp.pop(n)
			c_tmp.pop(n)
			d_tmp.pop(n)

	site_list   = s_tmp
	K_list      = k_tmp
	year_list   = y_tmp
	sName_list  = n_tmp
	sColor_list = c_tmp
	doy_list    = d_tmp

	print site_list, K_list, year_list, nlist, sColor_list

#======================================
#===== make list for variables roop
def make_list(N):

	acc1,acc2,acc3 = [],[],[]

	for i in range(N):
		acc1.append(i)
		acc2.append(i)
		acc3.append(i)

	return acc1,acc2,acc3

#======================================
#===== set path & read model data
def rd_csv(indir,site,year,FD):
	import datetime
	Tab_list = ['Date','mlthr2','mlthr3','mlthr4']

	infile = '%s/mlthr_%d_%s_%s.csv'%(indir,year,site,FD)

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data,doy = -999,-999
		return data,doy

	#=== read csv data
	N = Tab_list.index(Mlt)
	data = df[Tab_list[N]].values.tolist()
	data = data[1::]

	Date = df[Tab_list[0]].values.tolist()
	Date = Date[1::]

	doy = []
	for i in range(len(Date)):
		tmp = str(Date[i]).replace(' ','')
		tmp = datetime.datetime.strptime(tmp,'%Y%m%d%H%M%S')
		doy.append((tmp - datetime.datetime(year,1,1,12)).days + 1)  # cal doy

	return data,doy

#======================================
#===== cal algal cell concentration
def cal_acc(mlthr,FD,K,site,exp):

	if mlthr == -999:
		print "%s (%s) no data"%(exp,FD)
		acc_all,rab_all = -999,-999
		return acc_all,rab_all

	print "%s start (%s)"%(exp,FD)

	#=== ensemble parameters
	if exp == 'X0exp':
		acc_all = copy.deepcopy(X0_list)
		rab_all = copy.deepcopy(acc_all)
		for x in range(len(X0_list)):
			acc_all[x],rab_all[x] = logi(mlthr,X0_list[x],0.0180,site,K)

	elif exp == 'Uexp':
		acc_all = copy.deepcopy(Gr_list)
		rab_all = copy.deepcopy(acc_all)
		for g in range(len(Gr_list)):
			acc_all[g],rab_all[g] = logi(mlthr,6.33,Gr_list[g],site,K)
	elif exp == 'Fdexp':
		acc_all,rab_all = logi(mlthr,aexp_X0,0.0180,site,K)

	return acc_all,rab_all

#======================================
#===== cal duration of the algal blooming
def cal_dab(acc,rab,site,exp):

	if site == 'Livingston' and rab == -999:
		dab_min,dab_max = -999,-999
		return dab_min,dab_max

	if exp == 'X0exp':
		dab_min = copy.deepcopy(X0_list)
		dab_max = copy.deepcopy(X0_list)

		for x in range(len(X0_list)):
			dab = [i for i, val in enumerate(acc[x].tolist()) if val > 5e5]
			if not dab:
				dab_min[x] = rab[x]
				dab_max[x] = rab[x]
			else:
				dab_min[x] = np.min(dab)
				dab_max[x] = np.max(dab)
				if site in ['Philistine','Tyndall']:
					dab_min[x] = dab_min[x] + 179
					dab_max[x] = dab_max[x] + 179

	elif exp == 'Uexp':
		dab_min = copy.deepcopy(Gr_list)
		dab_max = copy.deepcopy(Gr_list)

		for g in range(len(Gr_list)):
			dab = [i for i, val in enumerate(acc[g].tolist()) if val > 5e5]
			if not dab:
				dab_min[g] = rab[g]
				dab_max[g] = rab[g]
			else:
				dab_min[g] = np.min(dab)
				dab_max[g] = np.max(dab)
				if site in ['Philistine','Tyndall']:
					dab_min[g] = dab_min[g] + 179
					dab_max[g] = dab_max[g] + 179

	elif exp == 'Fdexp':
		dab_min = -999
		dab_max = -999

	return dab_min,dab_max

#======================================
#===== logistic model
def logi(mlthr,X0,Gr,site,K):

	acc = np.zeros(len(mlthr))

	#=== cal algal cell conc.
	for t in range(len(mlthr)):
		acc[t] = K / ( 1 + (( K - X0 ) / X0 ) * ( np.exp(1) ** ( Gr * ( float(mlthr[t]) * -1 ))))

	#=== cal reaching time to the algal blooming
	rab = [i for i, x in enumerate(acc) if x > 5.e5]

	if not rab:
		# no appearance of the algal blooming (<5.e5) 
		rab = acc.tolist().index(np.max(acc))
	else:
		# algal blooming at first doy
		rab = np.min(rab)

	if site in ['Philistine','Tyndall']:
		rab = rab + 179

	#print rab

	return acc,rab

#======================================
#===== plot figure
def plot(outdir,site_list,year_list,d1_list,d2_list,d3_min_list,d3_max_list,FD,exp):

	if exp in ['X0exp']:
		xList = X0_list
	elif exp in ['Uexp']:
		xList = Gr_list
	elif exp == 'Fdexp':
		X = FD_List
		if FD == 6.33:
			X0 = 0
		elif FD == 694:
			X0 = 1
		if FD_List[1] == 'wfdei':
			label1 = "GSWP3-FD" #FD_List[0]
			label2 = "WFDEI" #FD_List[1]
			label3 = "CRUJRA" #FD_List[2]
		else:
			label1 = FD_List[0]
			label2 = FD_List[1]
			label3 = FD_List[2]

	#=== set plot param
	fig, axes = plt.subplots(1, 1, figsize=(11,10))
	plt.subplots_adjust(right=0.85) # adjust plot space

	#=== plot model result
	if exp in ['X0exp','Uexp']:
 		for n in range(len(site_list)):
			
			X = copy.deepcopy(xList)
			X_dot = copy.deepcopy(xList)
			d2_dot = copy.deepcopy(d2_list[n][:])

			for x in range(len(xList)):
				if d2_list[n][x] == d3_max_list[n][x]:
					d2_list[n][x] = np.nan
					d3_min_list[n][x] = np.nan 
					d3_max_list[n][x] = np.nan 
					X[x] = np.nan

			axes.plot(X,d2_list[n][:],color=sColor_list[n],lw=2,label=sName_list[n])
			axes.plot(X_dot,d2_dot,color=sColor_list[n],lw=2,linestyle="--")
		plt.legend(loc='upper right', bbox_to_anchor=(1.2,1.0),fontsize=16)

	elif exp in ['Fdexp']:
		# box plot
		bp = axes.boxplot(d2_list,patch_artist=True,labels=sName_list, zorder=1)

		for b,c in zip(bp['boxes'], sColor_list):
			b.set(color='black',linewidth=1)	# outline color
			b.set_facecolor(c)	# box color	(patch_artist=True is needed)

		# obs plot
 		for n in range(len(site_list)):
			axes.plot(n+1,doy_list[n],color="black",marker="o")


	#=== set label
	if exp == 'X0exp':
		axes.tick_params(axis='x',labelsize=16)
		axes.tick_params(axis='y',labelsize=16)
		axes.set_ylabel('Reaching time to the algal bloom\n(RTAB, Day of the Year)',fontsize=20)
		axes.set_xlabel('Initial cell concentration\n(cells $\mathregular{m^{-2}}$)',fontsize=20)
		axes.set_ylim(ymin=100,ymax=366)
		axes.set_xlim(xmin=1)
		axes.set_xlim(xmax=1000)
		axes.set_xscale("log")
		axes.text(1.1,110,"(a)",fontsize=22)
		figName = 'stest_%s_%dsites_%s_%s'%(Mlt,len(sName_list),exp,FD)
	elif exp == 'Uexp':
		axes.tick_params(axis='x',labelsize=16)
		axes.tick_params(axis='y',labelsize=16)
		axes.set_ylabel('Reaching time to the algal bloom\n(RTAB, Day of the Year)',fontsize=20)
		axes.set_xlabel('Algal growth rate\n($\mathregular{hour^{-1}}$)',fontsize=20)
		axes.set_ylim(ymin=100,ymax=366)
		axes.set_xlim(xmin=0.01)
		axes.set_xlim(xmax=0.025)
		axes.text(0.0102,110,"(b)",fontsize=22)
		figName = 'stest_%s_%dsites_%s_%s'%(Mlt,len(sName_list),exp,FD)
	elif exp == 'Fdexp':
		axes.tick_params(axis='x',labelsize=14)
		axes.tick_params(axis='y',labelsize=16)
		axes.set_ylabel('Reaching time to the algal bloom\n(RTAB, Day of the Year)',fontsize=20)
		axes.set_ylim(ymin=100,ymax=366)

		axes.axvline(x=12.5,color='black',ls=':')
		axes.text(13.25,115,"SH",fontsize=18)
		axes.text(3.7,115,"NH (Northern Hemisphere)",fontsize=18)

		if aexp_X0 == 6.33:
			figNm = "def"
			axes.text(1,345,"(a)",fontsize=22)
		else:
			aexp_X0L = [1,10,100,1000]
			aexp_X0L2 = ["b","c","d","e"]
			figNm = aexp_X0L2[aexp_X0L.index(aexp_X0)]
			axes.text(1,345,"(%s)"%figNm,fontsize=22)


		axes.text(1.5,325,"$\it{X_{0}}$ = %s cells $\mathregular{m^{-2}}$"%aexp_X0,fontsize=19)
		axes.text(1.5,310,"$\it{\mu}$ = 0.018 $\mathregular{hour^{-1}}$",fontsize=19)
		figName = 'stest_%s_%dsites_%s_%s'%(Mlt,len(sName_list),exp,figNm)


#===

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s.png'%(outdir,figName),bbox_inches="tight",dpi=300)
		print 'save %s/%s.png'%(outdir,figName)
	else:
		print '%s/%s.png'%(outdir,figName)
		plt.show()
	plt.close()

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <yonuma613@gmail.com>'
__version__ = '1.2.0'
__date__ = 'Dec. 2021'
