import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import copy

#======================================
#========== set param
figSave = 'off'

#=== LRexp:fig5b
Exp = 'LRexp'
X0 = 6.33 # initial cell concentration
Gr = 0.018 # algal growth rate

FD = 'wfdei'	# forcing

Mlt = 'mlthr4'

#y_min, y_max = 1e0, 1e11 	# log scale (X=694)

#=== temperature lapse rate parameters
lr_list = ['-5.0e-03','-5.5e-03','-6.0e-03','-6.5e-03','-7.0e-03','-7.5e-03','-8.0e-03']

#======================================
#========== main
def main():
	#===== set path
	indir = '../data'
	outdir = '../figure'

	#===== set name list
	set_list(Exp)

	data = np.zeros((len(site_list),len(lr_list),366)) 
	acc = np.zeros((len(site_list),len(lr_list),366)) 
	rab = np.zeros((len(site_list),len(lr_list))) 
	rab_flg = np.zeros((len(site_list),len(lr_list))) 
	dab_min = np.zeros((len(site_list),len(lr_list))) 
	dab_max = np.zeros((len(site_list),len(lr_list))) 

	#=== read melt hour (csv) & cal algal cell conc.
	for s in range(len(site_list)):
		for lr in range(len(lr_list)):
			print 'cal %s (%d) lr=%s'%(site_list[s],year_list[s],lr_list[lr])
			data[s,lr,:] = rd_csv(indir,site_list[s],year_list[s],lr_list[lr])
			acc[s,lr,:],rab[s,lr],rab_flg[s,lr] = cal_acc(data[s,lr,:],FD,K_list[s],site_list[s])
		dab_min[s,:],dab_max[s,:] = cal_dab(acc[s,:,:],rab[s,:],site_list[s])

	plot(outdir,site_list,year_list,acc,rab,rab_flg,dab_min,dab_max,FD)

#======================================
#===== set list for variables roop
def set_list(exp):
	global slist, klist, ylist, nlist, clist, dlist
	global site_list, K_list, year_list, sName_list, sColor_list, doy_list


	slist = ['Feiringbreen','QA_SB','Werenskioldbreen','Mittivakkat','Gulkana','No.33','Harding',
		 'Akkem','Tiefenbach','UM_No1','TigoaPass','TA_Rai','Philistine','Tyndall','Livingston']
	klist = [2.0e8         ,3.5e7  ,5.9e9             ,1.8e8        ,5.1e7    ,6.9e6    ,6.7e9  ,
		 2.2e6  ,5.0e8       ,1.3e6   ,1.3e7      ,1.1e8   ,1.2e7       ,6.0e5    ,4.9e8       ]
	ylist = [2013          ,2014   ,2004              ,2012         ,2001     ,2012     ,2000   ,
		 2002   ,2008        ,2013    ,2000       ,2012    ,1998        ,1999     ,2015        ]
	nlist = ['SV-S'        ,'GL-Q','SV-W'             ,'GL-A'       ,'AK-E'    ,'SB-S'  ,'AK-K' ,
		 'AT-A' ,'EU-T'      ,'TN-U'  ,'NA-S'     ,'JP-T'  ,'NZ-A'      ,'PG-T'   ,'AP-L']
	clist = ['red'         ,'sienna','orange'         ,'gold'   ,'yellowgreen','lime'   ,'mediumturquoise' ,
		 'cyan' ,'dodgerblue','lightsteelblue','navy' ,'darkviolet','magenta'   ,'pink'   ,'dimgray' ]
	dlist = [220           ,214   ,202                ,192          ,218      ,199      ,213   ,
		 199    ,255         ,238     ,200        ,167     ,358         ,319      ,26        ]

	s_tmp = copy.deepcopy(slist)
	k_tmp = copy.deepcopy(klist)
	y_tmp = copy.deepcopy(ylist)
	n_tmp = copy.deepcopy(nlist)
	c_tmp = copy.deepcopy(clist)
	d_tmp = copy.deepcopy(dlist)

	if exp == 'Iexp':
		rmlist = [0,0,0,1,0,0,0,0,0,0,0,0,0,0,1]
	elif exp == 'Gexp':
		rmlist = [0,0,0,1,0,1,0,0,1,0,0,0,1,0,1]
	elif exp in ['Aexp','LRexp']:
		rmlist = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,1]

	for n in range(len(rmlist)-1,-1,-1):
		if rmlist[n] == 1:
			s_tmp.pop(n)
			k_tmp.pop(n)
			y_tmp.pop(n)
			n_tmp.pop(n)
			c_tmp.pop(n)
			d_tmp.pop(n)

	site_list   = s_tmp
	K_list      = k_tmp
	year_list   = y_tmp
	sName_list  = n_tmp
	sColor_list = c_tmp
	doy_list    = d_tmp

	print site_list, K_list, year_list, nlist, sColor_list

#======================================
#===== set path & read model data
def rd_csv(indir,site,year,lr):
	import datetime
	import calendar

	Tab_list = ['Date','mlthr2','mlthr3','mlthr4']

	infile = '%s/LR%s_exp/csv/mlthr_%d_%s_%s.csv'%(indir,lr,year,site,FD)

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data,doy = -999,-999
		return data

	#=== read csv data
	N = Tab_list.index(Mlt)
	data = df[Tab_list[N]].values.tolist()[1:]	# remove unit
	Date = df[Tab_list[0]].values.tolist()[1:]	# remove unit

	if site in ['Philistine','Tyndall']:
		if not calendar.isleap(year):
			tmp2 = 181
		else:
			tmp2 = 182
		tmp = np.zeros((tmp2))
		tmp[:] = np.nan
		data = np.append(tmp,data)

	if not calendar.isleap(year):
		data = np.append(data,np.nan) # t 365 -> 366

	return data

#======================================
#===== cal algal cell concentration
def cal_acc(mlthr,FD,K,site):

	if -999 in mlthr:
		print "%s (%s) no data"%(exp,FD)
		acc_all,rab_all = -999,-999
		return acc_all,rab,rab_flg

	#=== logistic model
	acc_all,rab,rab_flg = logi(mlthr,X0,Gr,site,K)

	return acc_all,rab,rab_flg

#======================================
#===== cal duration of the algal blooming
def cal_dab(acc,rab,site):

	if site == 'Livingston' and -999 in rab:
		dab_min,dab_max = -999,-999
		return dab_min,dab_max

	dab_min = copy.deepcopy(lr_list)
	dab_max = copy.deepcopy(lr_list)

	for x in range(len(lr_list)):
		dab = [i for i, val in enumerate(acc[x].tolist()) if val > 5e5]
		if not dab:
			dab_min[x] = rab[x]
			dab_max[x] = rab[x]
		else:
			dab_min[x] = np.min(dab)
			dab_max[x] = np.max(dab)

	return dab_min,dab_max

#======================================
#===== logistic model
def logi(mlthr,X0,Gr,site,K):

	acc = np.zeros(len(mlthr))

	#=== cal algal cell conc.
	for t in range(len(mlthr)):
		acc[t] = K / ( 1 + (( K - X0 ) / X0 ) * ( np.exp(1) ** ( Gr * ( float(mlthr[t]) * -1 ))))

	#=== cal reaching time to the algal blooming
	rab = [i for i, x in enumerate(acc) if x > 5.e5]

	if not rab:
		# no appearance of the algal blooming (<5.e5) 
		rab = acc.tolist().index(np.nanmax(acc))
		rab_flg = 0
		print np.nanmax(acc),acc.tolist().index(np.nanmax(acc))
	else:
		# algal blooming at first doy
		rab = np.min(rab)
		rab_flg = 1
		print rab,acc[int(rab)]

	return acc,rab,rab_flg

#======================================
#===== plot figure
def plot(outdir,site_list,year_list,acc,rab,rab_flg,dab_min,dab_max,FD):

	lr_list_f = [-5.0e-03,-5.5e-03,-6.0e-03,-6.5e-03,-7.0e-03,-7.5e-03,-8.0e-03]

	#=== set plot param
	fig, axes = plt.subplots(1, 1, figsize=(11,10))
	plt.subplots_adjust(right=0.85) # adjust plot space

	#=== plot model result
	# box plot
	rab_list = []
	for s in range(len(site_list)):
		rab_list.append(rab[s,:])		

	bp = axes.boxplot(rab_list,patch_artist=True,labels=sName_list, zorder=1, sym="")

	for b,c in zip(bp['boxes'], sColor_list):
		b.set(color='black',linewidth=1)	# outline color
		b.set_facecolor(c)	# box color	(patch_artist=True is needed)

	# obs plot
 	for s in range(len(site_list)):
		axes.plot(s+1,doy_list[s],color="black",marker="o")

	#=== set label
	axes.tick_params(axis='x',labelsize=14)
	axes.tick_params(axis='y',labelsize=16)

	axes.set_ylabel('Reaching time to the algal bloom\n(RTAB, Day of the Year)',fontsize=20)
	axes.set_ylim(ymin=100,ymax=366)

	axes.axvline(x=12.5,color='black',ls=':')
	axes.text(13.25,115,"SH",fontsize=18)
	axes.text(3.7,115,"NH (Northern Hemisphere)",fontsize=18)

	axes.text(1,345,"(b)",fontsize=22)
	figName = 'stest_%s_%dsites_lrexp_%s'%(Mlt,len(sName_list),FD)

	axes.text(1.5,325,"$\it{X_{0}}$ = %s cells $\mathregular{m^{-2}}$"%str(X0),fontsize=19)
	axes.text(1.5,310,"$\it{\mu}$ = %s $\mathregular{hour^{-1}}$"%str(Gr),fontsize=19)
#===
	outdir = '%s'%outdir

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s.png'%(outdir,figName),bbox_inches="tight",dpi=300)
		print 'save %s/%s.png'%(outdir,figName)
	else:
		print '%s/%s.png'%(outdir,figName)
		plt.show()
	plt.close()

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'Dec. 2021'
