#!/usr/bin/env python2.7
"""
draw global map using netCDF file
"""
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import os
import warnings
warnings.filterwarnings('ignore')
from mpl_toolkits.axes_grid1 import make_axes_locatable

figSave = 'off'

#===== Var
Var = 'saglb4'
FD = 'wfdei'	# gswp3,wfdei,crujra

Year,eYear = 1980,2014

#===== option
fcolor = 'lightgrey'		# ocean & snow free area color
colors = ['white','aliceblue','lightcyan','paleturquoise','mediumturquoise','mediumaquamarine','darkorange','red']

#======================================
#========== main
def main():
	#=== set path
	infile = '../data/saglb4_matsiro6_198001-201412_clim_%s.nc'%FD
	outDir = '../figure'
	print'### read data ###'
	print infile
	d = read_data(infile,Var)
	print d.shape
	
	#=== Cal monthly mean & plot
	print 'Monthly mean'
	setmlist(d)
	dataM_all = list(range(12))
	for mon in range(len(mList)):
		print '%s mean'%mList[mon]
		dataM_all[mon] = monmean(d,mon)
	print'### draw figures ###'
	drawmap(dataM_all,outDir)

#======================================
#===== read file
def read_data(infile,Var):
	from netCDF4 import Dataset
	global Unit,lon_all,lat_all,lev,log

	#=== read netCDF file
	nc   = Dataset(infile, mode='r')
	data  = nc.variables[Var][:]	

	Unit  = nc.variables[Var].units	
	lat_all = nc.variables['lat'][:]
	lon_all = nc.variables['lon'][:]

	if Unit in ['cells/m2','cells/m**2']:
		lev,log = np.logspace(0,8,num=9), True  # 10^0~10^8

	return data

#======================================
#===== Set mon list for monthly mean
def setmlist(data):
	global mList,t,t4
	mList = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']
	(nT,nY,nX) = data.shape
	if nT == 12:
		print 'Monthly input data'
		t  = [0,1,2,3,4,5,6,7,8,9,10,11,12]
		t4 = [0,1,2,3,4,5,6,7,8,9,10,11,12]

#======================================
#===== Cal monthly mean
def monmean(data,mon):
	monData = data[mon,:,:]
		
	if t[-1] == 12 or t[-1] == 14: 
		monMean = monData
	else:	
		monMean = np.mean(monData,axis=0)

	return monMean

#======================================
#===== draw map plot
def drawmap(dataMa,outDir):
	import matplotlib.ticker as tic

	#=== set plot param
	fig, axes = plt.subplots(4, 3, figsize=(15,11))
	plt.rcParams['font.size'] = 13

	i,j = 0,0
	for mon in range(len(mList)):
		print i,j

		(x,y,mmap) = map0(lon_all,lat_all,0,360,-90,90,axes,i,j)

		if log == True:
			cs = axes[i][j].contourf(x,y,dataMa[mon],levels=lev,locator=tic.LogLocator(),colors=colors)
			#== plot pixel
			mmap.drawmapboundary(fill_color=fcolor) # for imshow
			axes[i][j].text(10,-60,mList[mon],color="black",fontsize=20)
		else:
			cs = axes[i][j].contourf(x,y,dataMa,levels=lev,cmap=cmap,extend='both')

		j = j + 1
		if j == 3:
			i = i + 1
			j = 0

	#===== Cordinate Colorbar
	divider = make_axes_locatable(axes[0][0])
	cbar = plt.colorbar(cs, orientation='horizontal',cax=fig.add_axes([0.025,-0.03,0.95,0.020]))  #[x,y,l,w]
	cbar.ax.tick_params(labelsize=16)
	cbar.set_label('Algal cell concentration (cells $\mathregular{m^{-2}}$)',fontsize=20)

	cbar.ax.set_xticklabels(['$\mathregular{10^{0}}$','$\mathregular{10^{1}}$','$\mathregular{10^{2}}$'
	,'$\mathregular{10^{3}}$','$\mathregular{10^{4}}$','$\mathregular{10^{5}}$'
	,'$\mathregular{10^{6}}$','$\mathregular{10^{7}}$','$\mathregular{10^{8}}$'])

	plt.tight_layout()
	figN = '%s_%d-%d_%s'%(Var,Year,eYear,FD)

	#=== save figure 
	if figSave=='on':
		if os.path.isdir('%s'%outDir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outDir,figN),format='png',bbox_inches="tight",dpi=300)
		print 'save figure %s/%s.png'%(outDir,figN)
	else:
		print 'Out dir = %s'%outDir
		print 'Figure name = %s.png'%figN
		plt.show()

	plt.close()

#======================================
#===== draw map (global)
def map0(lon,lat,lonMin,lonMax,latMin,latMax,axes,i,j):

	# create Basemap instance.
	mmap=Basemap(projection='cyl',llcrnrlon=lonMin,urcrnrlon=lonMax,llcrnrlat=latMin,urcrnrlat=latMax,resolution='l',ax=axes[i][j])

	# draw coastlines, state and country boundaries, edge of map.
	mmap.drawcoastlines(color='k',linewidth=0.8)

	if j == 0 and i == 3:
		label = [1,0,0,1]
	elif j == 0:
		label = [1,0,0,0]
	elif i == 3:
		label = [0,0,0,1]
	else:
		label = [0,0,0,0]

	mmap.drawparallels(np.arange(-90,90+1,30),color='0.5',linewidth=0,labels=label)
	mmap.drawmeridians(np.arange(30,361,60),color='0.5',linewidth=0,labels=label)

	# find x,y of map projection grid.
	x, y = mmap(lon, lat)
	return x,y,mmap

#======================================

if __name__=='__main__':
    main()

__author__ = 'Yukihiko Onuma <yonuma613@gmail.com>'
__version__ = '1.0.0'
__date__ = 'Mar. 2021'
