#!/usr/bin/env python2.7
"""
draw global map using netCDF file
"""
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import os
import warnings
warnings.filterwarnings('ignore')
from mpl_toolkits.axes_grid1 import make_axes_locatable
import sys

figSave = 'off'

#===== Var
Var = 'rtab'
Fd = 'wfdei'	# wfdei only
Reso = '05deg'

Year,eYear = 1980,2014

X0_l = [60,6.33,694]	# def, min, max
Gr_l = [0.017,0.016,0.018]
K_l = [3.5e7,6.0e5,6.7e9]
#Fd_l = ['gswp3','wfdei','crujra']

#===== set param
X0_1a,Gr_1a,K_1a = X0_l[1],Gr_l[0],K_l[0]
X0_1b,Gr_1b,K_1b = X0_l[2],Gr_l[0],K_l[0]
X0_2a,Gr_2a,K_2a = X0_l[0],Gr_l[1],K_l[0]
X0_2b,Gr_2b,K_2b = X0_l[0],Gr_l[2],K_l[0]
X0_3a,Gr_3a,K_3a = X0_l[0],Gr_l[0],K_l[1]
X0_3b,Gr_3b,K_3b = X0_l[0],Gr_l[0],K_l[2]

#===== option
fcolor = 'lightgrey'		# ocean & snow free area color
#fcolor = 'lightgray'		# ocean & snow free area color

#======================================
#========== main
def main():
	#=== set path
	indir = '../data' 
	outdir = '../figure' 

	f_1a = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_1a,Gr_1a,K_1a/1e5,Fd)
	f_1b = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_1b,Gr_1b,K_1b/1e5,Fd)
	f_2a = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_2a,Gr_2a,K_2a/1e5,Fd)
	f_2b = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_2b,Gr_2b,K_2b/1e5,Fd)
	f_3a = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_3a,Gr_3a,K_3a/1e5,Fd)
	f_3b = '%s/%s_matsiro6_%d-%d_%s_%s_%.0f_%s.nc'%(indir,Var,Year,eYear,X0_3b,Gr_3b,K_3b/1e5,Fd)

	print'### read data ###'
	print f_1a
	print f_1b
	print f_2a
	print f_2b
	print f_3a
	print f_3b
	d_1a = read_data(f_1a,Var)
	d_1b = read_data(f_1b,Var)
	d_2a = read_data(f_2a,Var)
	d_2b = read_data(f_2b,Var)
	d_3a = read_data(f_3a,Var)
	d_3b = read_data(f_3b,Var)
	print d_1a.shape,d_1b.shape
	print d_2a.shape,d_2b.shape
	print d_3a.shape,d_3b.shape

	#=== 

	#=== Cal yearly mean & plot
	print 'Yearly mean'
	d_1a = np.nanmean(d_1a,axis=0)
	d_1b = np.nanmean(d_1b,axis=0)
	d_2a = np.nanmean(d_2a,axis=0)
	d_2b = np.nanmean(d_2b,axis=0)
	d_3a = np.nanmean(d_3a,axis=0)
	d_3b = np.nanmean(d_3b,axis=0)

	print'### draw figures ###'
	drawmap(d_1a,d_1b,d_2a,d_2b,d_3a,d_3b,outdir)

#======================================
#===== read file
def read_data(infile,Var):
	from netCDF4 import Dataset
	global Unit,lon_all,lat_all,lev1,lev2,log

	#=== read netCDF file
	nc   = Dataset(infile, mode='r')
	data  = nc.variables[Var][:]	

	Unit  = nc.variables[Var].units	
	lat_all = nc.variables['lat'][:]
	lon_all = nc.variables['lon'][:]

	if Unit in ['cells/m2','cells/m**2','hours']:
		Unit = "cells/m2"
		lev,log = np.logspace(0,9,num=10), True  # 10^0~10^8
	elif Unit in ['Day of the Year']:
		lev1,log = np.arange(0,420.1,10), False
		lev2 = np.arange(-50,0.1,2.5)

	return data

#======================================
#===== draw map plot
def drawmap(d_1a,d_1b,d_2a,d_2b,d_3a,d_3b,outDir):
	import matplotlib.ticker as tic

	#=== set plot param
	fig, axes = plt.subplots(3, 3, figsize=(16,9))
	plt.rcParams['font.size'] = 13

	d_list = [d_1a,d_1b,d_1b-d_1a,d_2a,d_2b,d_2b-d_2a,d_3a,d_3b,d_3b-d_3a]
	d_lab = ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)']

	i,j = 0,0
	for ij in range(9):
		print i,j

		(x,y,mmap) = map0(lon_all,lat_all,0,360,-90,90,axes,i,j)
		
		if j == 2: 
			cs2 = axes[i][j].contourf(x,y,d_list[ij],levels=lev2,cmap='Reds_r',extend='both')
		else:
			cs1 = axes[i][j].contourf(x,y,d_list[ij],levels=lev1,cmap='jet_r',extend='both')
			if ij == 0:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[1],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[0],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $3.5x10^7$",color="black",fontsize=15)
			elif ij == 1:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[2],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[0],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $3.5x10^7$",color="black",fontsize=15)
			elif ij == 3:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[1],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $3.5x10^7$",color="black",fontsize=15)
			elif ij == 4:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[2],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $3.5x10^7$",color="black",fontsize=15)
			elif ij == 6:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[0],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[1],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $6.0x10^5$",color="black",fontsize=15)
			elif ij == 7:
				axes[i][j].text(185,-22,"$\it{X_{0}}$ = %s"%X0_l[0],color="black",fontsize=15)
				axes[i][j].text(185,-42,"$\it{\mu}$ = %s"%Gr_l[0],color="black",fontsize=15)
				#axes[i][j].text(185,-62,"$\it{K}$ = %.1e"%K_l[2],color="black",fontsize=15)
				axes[i][j].text(185,-62,"$\it{K}$ = $6.7x10^9$",color="black",fontsize=15)

		mmap.drawmapboundary(fill_color=fcolor)
		axes[i][j].text(10,-60,d_lab[ij],color="black",fontsize=20)


		j = j + 1
		if j == 3:
			i = i + 1
			j = 0

	#===== Cordinate Colorbar
	divider = make_axes_locatable(axes[0][0])
	cbar = plt.colorbar(cs1, orientation='horizontal',cax=fig.add_axes([0.020,-0.015,0.95,0.020]))  #[x,y,l,w]
	cbar.ax.tick_params(labelsize=16)

	cbar = plt.colorbar(cs2, orientation='horizontal',cax=fig.add_axes([0.020,-0.075,0.95,0.020]))  #[x,y,l,w]
	cbar.ax.tick_params(labelsize=16)
	cbar.set_label('Reaching time to the algal bloom (RTAB, Day of the Year from Jan. 1st)',fontsize=20)

	plt.tight_layout()
	figN = '%s_%d-%d_%s_subp'%(Var,Year,eYear,FD)

	#=== save figure 
	if figSave=='on':
		if os.path.isdir('%s'%outDir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outDir,figN),format='png',bbox_inches="tight",dpi=300)
		print 'save figure %s/%s.png'%(outDir,figN)
	else:
		print 'Out dir = %s'%outDir
		print 'Figure name = %s.png'%figN
		plt.show()

	plt.close()

#======================================
#===== draw map (global)
def map0(lon,lat,lonMin,lonMax,latMin,latMax,axes,i,j):

	# create Basemap instance.
	mmap=Basemap(projection='cyl',llcrnrlon=lonMin,urcrnrlon=lonMax,llcrnrlat=latMin,urcrnrlat=latMax,resolution='l',ax=axes[i][j])

	# draw coastlines, state and country boundaries, edge of map.
	mmap.drawcoastlines(color='k',linewidth=0.8)

	if j == 0 and i == 2:
		label = [1,0,0,1]
	elif j == 0:
		label = [1,0,0,0]
	elif i == 2:
		label = [0,0,0,1]
	else:
		label = [0,0,0,0]

	mmap.drawparallels(np.arange(-90,90+1,30),color='0.5',linewidth=0,labels=label)
	mmap.drawmeridians(np.arange(30,361,60),color='0.5',linewidth=0,labels=label)

	# find x,y of map projection grid.
	x, y = mmap(lon, lat)
	return x,y,mmap

#======================================

if __name__=='__main__':
    main()

__author__ = 'Yukihiko Onuma <yonuma613@gmail.com>'
__version__ = '1.2.0'
__date__ = 'Dec. 2021'
