#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys

#======================================
#========== set param
figSave = 'off'

Var = 'ssrd'

if Var == 'gdt':
	y_min, y_max = 250, 300	# temp
elif Var in ['ssrd','slrd']:
	y_min, y_max = 0, 500	# radiation (short or long)
elif Var == 'gdq':
	y_min, y_max = 0,0.02	# humidity
elif Var == 'gdu':
	y_min, y_max = 0,15 	# wind speed
elif Var == 'gprct':
	y_min, y_max = 0, 100	# rain+snow fall
elif Var == 'gsnwl':
	y_min, y_max = 0, 50	# snow fall

doy_min, doy_max = 0,365	# north hemisphere

#======================================
#========== set list
FD_List = ['gswp3','wfdei','crujra']	# forcing

site_list = ['Feiringbreen','QA_SB','Werenskioldbreen','Mittivakkat','Gulkana','No.33','Harding',
		 'Akkem','Tiefenbach','UM_No1','TigoaPass','TA_Rai','Philistine','Tyndall','Livingston']
year_list = [2013          ,2014   ,2004              ,2012         ,2001     ,2012   ,2000   ,
		 2002   ,2008        ,2013    ,2000       ,2012    ,1998        ,1999     ,2015        ]
sName_list =['SV-S'        ,'GL-Q','SV-W'             ,'GL-A'       ,'AK-E'   ,'SB-S' ,'AK-K' ,
		 'AT-A' ,'EU-T'      ,'XN-T'  ,'NA-S'     ,'JP-T'  ,'NZ-A'      ,'PG-T'   ,'AP-L']

#======================================
#========== main
def main():
	#===== set path
	indir = '../data'
	outdir = '../figure'

	#===== make list for sites loop
	d1,d2,d3 = make_list(len(site_list))	# FD1, FD2, FD3

	#=== read melt hour2,3,4 (csv)
	for n in range(len(site_list)):
		d1[n] =rd_csv(indir,site_list[n],year_list[n],FD_List[0])
		d2[n] =rd_csv(indir,site_list[n],year_list[n],FD_List[1])
		d3[n] =rd_csv(indir,site_list[n],year_list[n],FD_List[2])

	#= FDs data_L[FD][site]
	data_L = [d1,d2,d3]
	#=== figure
	plot(outdir,site_list,year_list,data_L,FD_List)

#======================================
#===== make list for variables roop
def make_list(N):

	acc1,acc2,acc3 = [],[],[]

	for i in range(N):
		acc1.append(i)
		acc2.append(i)
		acc3.append(i)

	return acc1,acc2,acc3

#======================================
#===== set path & read model data
def rd_csv(indir,site,year,FD):
	import datetime
	Tab_list = ['Date','gdt'  ,'ssrd' ,'slrd' ,'gdq'    ,'gdu'  ,'gprct'     ,'gsnwl']

	infile = '%s/atm_%d_%s_%s.csv'%(indir,year,site,FD)

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data,doy = -999,-999
		return data,doy

	#=== read csv data
	N = Tab_list.index(Var)
	data = df[Tab_list[N]].values.tolist()
	data = data[1::]
	for i in range(len(data)):
		if Var in ['gprct','gsnwl']:
			data[i] = float(data[i]) * 3600 * 24	# mm/s -> mm/day
		else:
			data[i] = float(data[i])

	return data

#======================================
#===== plot figure
def plot(outdir,site_list,year_list,data_list,FD):

	label1 = "GSWP3-FD" #FD[0]
	label2 = "WFDEI" #FD[1]
	label3 = "CRUJRA" #FD[2]

	#=== set plot param
	fig, axes = plt.subplots(5, 3, figsize=(16,14))
	plt.subplots_adjust(wspace=0.15,hspace=0.60) # adjust plot space

	#=== plot model result
	C = ["blue","green","red"]

	i,j = 0,0
	for n in range(len(site_list)):

		#== set x-axis
		axes[i][j].set_xlim(xmin=doy_min)
		axes[i][j].set_xlim(xmax=doy_max)
		xlabels = np.arange(doy_min,doy_max+1,30)
		axes[i][j].set_xticks(xlabels)
		axes[i][j].set_xlabel('Day of the Year (%d)'%year_list[n],fontsize=10)

		#== set y-axis
		if j == 0 and Var == 'gdt':
			axes[i][j].set_ylabel('Surface air\ntemperature (K)',fontsize=11)
		elif j == 0 and Var == 'ssrd':
			axes[i][j].set_ylabel('Downward shortwave\nradiation (W $\mathregular{m^{-2}}$)',fontsize=11)
		elif j == 0 and Var == 'slrd':
			axes[i][j].set_ylabel('Downward longwave\nradiation (W $\mathregular{m^{-2}}$)',fontsize=11)
		elif j == 0 and Var == 'gdq':
			axes[i][j].set_ylabel('Specific humidity\n(kg $\mathregular{kg^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gdu':
			axes[i][j].set_ylabel('10 m Wind speed\n(m $\mathregular{s^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gprct':
			axes[i][j].set_ylabel('Precipitation rate\n(mm $\mathregular{day^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gsnwl':
			axes[i][j].set_ylabel('Snowfall rate\n(mm $\mathregular{day^{-1}}$)',fontsize=11)

		axes[i][j].set_ylim(ymin=y_min)
		axes[i][j].set_ylim(ymax=y_max)
		axes[i][j].set_title(sName_list[n],fontsize=15)

		#== plot results (no data skip)
		print n
		if site_list[n] == 'Livingston' and FD in ['wfdei']:
			data1 = data_list[0][n]
			data2 = data_list[1][n]
			data3 = data_list[2][n]
			if Var == 'gdt':
				axes[i][j].axhline(y=273.15,color='grey',ls='--')
		elif site_list[n] == 'Livingston' and FD in ['gswp3','crujra']:
			data1 = 0
			data2 = 0
			data3 = 0
		else:
			data1 = data_list[0][n]
			data2 = data_list[1][n]
			data3 = data_list[2][n]
			if Var == 'gdt':
				axes[i][j].axhline(y=273.15,color='grey',ls='--')

		axes[i][j].plot(data1,color=C[0],lw=1,label=label1)
		axes[i][j].plot(data2,color=C[1],lw=1,label=label2)
		axes[i][j].plot(data3,color=C[2],lw=1,label=label3)

		j = j + 1
		if j == 3:
			i = i + 1
			j = 0

#===

	plt.legend(loc='lower center', bbox_to_anchor=(-0.65,-0.85),ncol=4,fontsize=14)

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s_15sites_3FD.png'%(outdir,Var),bbox_inches="tight",dpi=300)
		print 'save %s/%s_15sites_3FD.png'%(outdir,Var)
	else:
		print '%s/%s_15sites_3FD.png'%(outdir,Var)
		plt.show()
	plt.close()

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <yonuma613@gamil.com>'
__version__ = '1.0.0'
__date__ = 'Mar. 2021'
