import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import calendar

#======================================
#========== set param
figSave = 'off'

Tab = 'Other'
Var = 'gdt'

if Var == 'gdt':
	y_min, y_max = 250, 300	# temp
elif Var in ['ssrd','slrd']:
	y_min, y_max = 0, 500
elif Var == 'gdq':
	y_min, y_max = 0,0.02 
elif Var == 'gdu':
	y_min, y_max = 0,15 
elif Var == 'gprct':
	y_min, y_max = 0, 100
elif Var == 'gsnwl':
	y_min, y_max = 0, 50

Reso = 'v00'
doy_min, doy_max = 0,365	# north hemisphere

#======================================
#========== set list
lr_list = ['-5.0e-03','-5.5e-03','-6.0e-03','-6.5e-03','-7.0e-03','-7.5e-03','-8.0e-03']
lr_list_lb = ['$-5.0x10^{-3}$','$-5.5x10^{-3}$','$-6.0x10^{-3}$','$-6.5x10^{-3}$','$-7.0x10^{-3}$','$-7.5x10^{-3}$','$-8.0x10^{-3}$']

FD = 'wfdei'	# forcing

site_list = ['Feiringbreen','QA_SB','Werenskioldbreen','Mittivakkat','Gulkana','No.33','Harding',
		 'Akkem','Tiefenbach','UM_No1','TigoaPass','TA_Rai','Philistine','Tyndall','Livingston']
year_list = [2013          ,2014   ,2004              ,2012         ,2001     ,2012   ,2000   ,
		 2002   ,2008        ,2013    ,2000       ,2012    ,1998        ,1999     ,2015        ]
sName_list =['SV-S'        ,'GL-Q','SV-W'             ,'GL-A'       ,'AK-E'   ,'SB-S' ,'AK-K' ,
		 'AT-A' ,'EU-T'      ,'TN-U'  ,'NA-S'     ,'JP-T'  ,'NZ-A'      ,'PG-T'   ,'AP-L']

#======================================
#========== main
def main():
	#===== set path
	indir = '../data'
	outdir = '../figure'

	#===== read csv data
	print'### read obs data (csv) ###'
	
	#===== make list for sites loop
	d = np.zeros((len(site_list),len(lr_list),366))

	#=== read melt hour2,3,4 (csv)
	for s in range(len(site_list)):
		for lr in range(len(lr_list)):
			d[s,lr,:] = rd_csv(indir,site_list[s],year_list[s],lr_list[lr])

	print d.shape
	#=== figure
	plot(outdir,site_list,year_list,d,lr_list)

#======================================
#===== set path & read model data
def rd_csv(indir,site,year,lr):

	if site in ['Philistine','Tyndall']:
		infile = '%s/LR%s_exp/csv/atm_%d_%s_%s_met.csv'%(indir,lr,year,site,FD)
	else:
		infile = '%s/LR%s_exp/csv/atm_%d_%s_%s.csv'%(indir,lr,year,site,FD)

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999
		return data

	#=== read csv data
	Tab_list = ['Date','gdt','ssrd','slrd','gdq','gdu','gprct','gsnwl']
	data = df[Var].values.tolist()[1:]	# remove unit
	
	print "%s %d (%s) "%(site,year,FD),len(data)

	if not calendar.isleap(year):
		data = np.append(data,np.nan)

	#=== mm/s -> mm/day
	if Var in ['gprct','gsnwl']:
		for i in range(len(data)):
			data[i] = float(data[i]) * 3600 * 24

	return data

#======================================
#===== plot figure
def plot(outdir,site_list,year_list,data,lr_list):

	#=== set plot param
	fig, axes = plt.subplots(5, 3, figsize=(16,14))
	plt.subplots_adjust(wspace=0.15,hspace=0.60) # adjust plot space

	#=== plot model result
	C = rainbow(len(lr_list))	# color list

	i,j = 0,0
	for s in range(len(site_list)):

		#== set x-axis
		axes[i][j].set_xlim(xmin=doy_min)
		axes[i][j].set_xlim(xmax=doy_max)
		xlabels = np.arange(doy_min,doy_max+1,30)
		axes[i][j].set_xticks(xlabels)
		axes[i][j].set_xlabel('Day of the Year (%d)'%year_list[s],fontsize=10)

		#== set y-axis
		if j == 0 and Var == 'gdt':
			axes[i][j].set_ylabel('Surface air\ntemperature (K)',fontsize=11)
		elif j == 0 and Var == 'ssrd':
			axes[i][j].set_ylabel('Downward shortwave\nradiation (W $\mathregular{m^{-2}}$)',fontsize=11)
		elif j == 0 and Var == 'slrd':
			axes[i][j].set_ylabel('Downward longwave\nradiation (W $\mathregular{m^{-2}}$)',fontsize=11)
		elif j == 0 and Var == 'gdq':
			axes[i][j].set_ylabel('Specific humidity\n(kg $\mathregular{kg^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gdu':
			axes[i][j].set_ylabel('10 m Wind speed\n(m $\mathregular{s^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gprct':
			axes[i][j].set_ylabel('Precipitation rate\n(mm $\mathregular{day^{-1}}$)',fontsize=11)
		elif j == 0 and Var == 'gsnwl':
			axes[i][j].set_ylabel('Snowfall rate\n(mm $\mathregular{day^{-1}}$)',fontsize=11)

		axes[i][j].set_ylim(ymin=y_min)
		axes[i][j].set_ylim(ymax=y_max)
		axes[i][j].set_title(sName_list[s],fontsize=15)

		#== plot results 
		print s
		if Var == 'gdt':
			axes[i][j].axhline(y=273.15,color='grey',ls='--')

		for lr in range(len(lr_list)):
			axes[i][j].plot(data[s,lr,:],color=C[lr],lw=1,ls='-',label=lr_list_lb[lr])

		j = j + 1
		if j == 3:
			i = i + 1
			j = 0

#===

	plt.legend(loc='lower center', bbox_to_anchor=(-0.65,-0.85),ncol=4,fontsize=14)

	outdir = '%s'%(outdir)

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s_15sites_lrenm.png'%(outdir,Var),bbox_inches="tight",dpi=300)
		print 'save %s/%s_15sites_lrenm.png'%(outdir,Var)
	else:
		print '%s/%s_15sites_lrenm.png'%(outdir,Var)
		plt.show()
	plt.close()

#======================================
#===== rainbow color
def rainbow( nX ):
	c = ["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"]
	c_r = ["f","e","d","c","b","a","9","8","7","6","5","4","3","2","1","0"]
	c_list=[]
	for i in range(16):
	    x1 = "#ff"+c[i]
	    for j in range(16):
	        x2 = x1+c[j]+"00"
	        c_list.append(x2)
	for i in range(16):
	    x1 = "#"+c_r[i]
	    for j in range(16):
	        x2 = x1+c_r[j]+"ff00"
	        c_list.append(x2)
	for i in range(16):
	    x1 = "#00ff"+c[i]
	    for j in range(16):
	        x2 = x1+c[j]
	        c_list.append(x2)
	for i in range(16):
	    x1 = "#00"+c_r[i]
	    for j in range(16):
	        x2 = x1+c_r[j]+"ff"
	        c_list.append(x2)
	for i in range(16):
	    x1 = "#"+c[i]
	    for j in range(16):
	        x2 = x1+c[j]+"00ff"
	        c_list.append(x2)
	for i in range(16):
	    x1 = "#ff00"+c_r[i]
	    for j in range(16):
	        x2 = x1+c_r[j]
	        c_list.append(x2)
	
	col = sorted(set(c_list), key=c_list.index)

	col_list=[]
	for i in range(nX):
	    a = col[int(len(col)/nX) * i]
	    col_list.append(a)

	return col_list

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'Dec. 2021'
